# Copyright 2005 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""Hooks and hook management."""


class Hook:

    """Base class for hooks. Do not instantiate this, only subclass it."""

    def __init__(self):
        self.callbacks = []
        
    def add(self, callback):
        if callback not in self.callbacks:
            self.callbacks.insert(0, callback)
        
    def remove(self, callback):
        if callback in self.callbacks:
            self.callbacks.remove(callback)
        
    def __call__(self, *args):
        return self.do_callbacks(args)


class QueryHook(Hook):

    """A hook class for queries (object/None return values).
    
    All hooks of this type get some arguments, and return a value of None or a
    reference to an object, where None means "I don't know". The hook calls
    callbacks in the reverse order they were registered in until something
    other than None is returned. This is the return value of the hook.  If all
    callbacks return None, or there are no callbacks, the default value given
    to the constructor is returned.
    """
    
    def __init__(self, default_value):
        Hook.__init__(self)
        self.default_value = default_value

    def do_callbacks(self, args):
        for callback in self.callbacks:
            ret = callback(*args)
            if ret is not None:
                return ret
        return self.default_value


class ManipulationHook(Hook):

    """A hook for doing some manipulation of the state of the universe.
    
    Hooks of this type are used to manipulate the state of the universe,
    for example, to create a new list. The callbacks should return
    normally if the task has been completed, with a value of True if the
    next callback shouldn't be called and False otherwise (errors are
    signalled via exceptions, of course).
    
    """
    
    def do_callbacks(self, args):
        for callback in self.callbacks:
            if callback(*args):
                return


class Hooks:

    """Interface for storing all hooks."""

    def __init__(self):
        self.dict = {}
        self.docs = {}

    def add(self, name, object, doc):
        self.dict[name] = object
        self.docs[name] = doc

    def __getattr__(self, name):
        if name in self.dict:
            return self.dict[name]
        else:
            raise AttributeError("Unknown hook name " + name)


hooks = Hooks()

hooks.add("nanonano", 
          QueryHook(True), 
          "A dummy hook for testing.")

hooks.add("setup_option_parser",
          ManipulationHook(),
          """Set up eoc.options.parser (an optparse.OptionParser) by adding
          options to it. No arguments to callback.""")

hooks.add("handle_options",
          ManipulationHook(),
          """Handle options from the command line. Arguments are a mailing
          list manager, an options object as defined by the optoparse
          module, and a list of non-option arguments.""")
          
hooks.add("created_list_manager",
          ManipulationHook(),
          """This hook is called when a new MailingListManager has been
          created. The MLM is given as the argument.""")

hooks.add("find_lists",
          ManipulationHook(),
          """Find names of all lists. Append them to the list given as
          argument.""")

hooks.add("list_exists",
          QueryHook(True),
          """Does a list exist? Arguments are mailing list manager object,
          and name of list.""")

hooks.add("create_list",
          ManipulationHook(),
          """Create a new list on-disk. Arguments are mailing list manager
          object and mailing list object.""")

hooks.add("open_list",
          QueryHook(None),
          """Open an existing mailing list. Arguments are mailing list
          manager and name of list. Return a MailingList, or throw
          the ListDoesNotExist exception if it doesn't exist.""")

hooks.add("read_incoming",
          QueryHook(None),
          """Read an incoming message. No arguments. Return value must be a
          tuple consisting of the message as an email.Message object,
          the SMTP sender address, and the SMTP recipient address.""")

hooks.add("send_mail",
          ManipulationHook(),
          """Send mail to one or more recipients. Three arguments: 
          an email.Message object with the message, the address of the 
          SMTP sender, and a list of addresses for the recipients.""")
