# Copyright 2005 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""Mailing list."""


import md5


import eoc.exceptions


class BadListName(eoc.exceptions.EocException):

    """Exception thrown when list name is syntactically bad."""
    
    def __init__(self, listname):
        self.str = "'%s' does not contain '@'." % listname
        

class ListExistsAlready(eoc.exceptions.EocException):

    """Exception thrown when trying to create a list that already exists."""
    
    def __init__(self, listname):
        self.str = "List '%s' already exists." % listname 


class ListDoesNotExist(eoc.exceptions.EocException):

    """Exception thrown when trying to open a list that does not exist."""
    
    def __init__(self, listname):
        self.str = "List '%s' does not exist." % listname


class MailingList:

    """A mailing list."""
    
    def __init__(self, mlm, name):
        self._mlm = mlm
        self._name = name
        self.config = None
        self.subscriberdb = None
    
    def create(self):
        """Create the mailing list."""
        if "@" not in self._name:
            raise BadListName(self._name)
        if eoc.hooks.list_exists(self._mlm, self._name):
            raise ListExistsAlready(self._name)
        eoc.hooks.create_list(self._mlm, self)
        
    def process_incoming(self, msg, sender, parts):
        """Process an incoming message."""
        if not parts:
            eoc.hooks.send_mail(msg, 
                                self.command_address("ignore"), 
                                self.subscriberdb.list())
        else:
            self._mlm._commands[parts[0]](self._mlm, self, sender, 
                                          parts[1:], msg)

    def command_address(self, command, args=[]):
        """Create a command address."""
        (pling, plong) = self._name.split("@", 1)
        args = ["=".join(arg.split("@")) for arg in args]
        return "%s@%s" % ("-".join([pling, command] + args), plong)

    def compute_hash(self, command, args):
        """Compute the hash for a given address."""
        m = md5.new()
        m.update(self._mlm.secret())
        m.update(self.command_address(command, args))
        return m.hexdigest()

    def hashed_command_address(self, command, args=[]):
        """Create a command address with a hash."""
        hash = self.compute_hash(command, args)
        return self.command_address(command, args + [hash])

    def hashed_is_ok(self, command, args):
        """Is a hashed address OK? Last item in args is hash."""
        hash = self.compute_hash(command, args[:-1])
        return hash == args[-1]
