# Copyright 2005 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""Mailing list manager."""


import logging
import os


import eoc.defaults
import eoc.plugins
import eoc.ml


class MailingListManager:

    """The entire mailing list manager.
    
    A mailing list manager is represented on-disk by the "dot
    directory", by default ~/.enemies-of-carlotta, but this can be
    overriden with the "dotdir" argument to the constructor. The dot
    directory contains all the data the mailing list manager needs to
    function: a file called "secret" containing random bits used for
    authentication, and a subdirectory (with "@" in the name) for each
    mailing the manager manages. There may be other files there (perhaps
    created by the sysadmin). 

    """
    
    def __init__(self, dotdir=None):
        if dotdir is None:
            self._dotdir = self.default_dotdir()
        else:
            self._dotdir = dotdir
        self._commands = eoc.commands.AddressCommands()
        eoc.hooks.created_list_manager(self)

    def default_dotdir(self):
        """Return the name of the default dot directory."""
        return os.path.expanduser(eoc.defaults.dotdir)

    def create_dotdir(self):
        """Create the dot directory."""
        if not os.path.exists(self._dotdir):
            os.makedirs(self._dotdir)
            self.create_secret()

    def create_secret(self):
        """Create the file 'secret' with some random bits."""
        name = os.path.join(self._dotdir, "secret")
        if not os.path.exists(name):
            f = file("/dev/urandom", "r") # XXX Linuxism
            secret = f.read(32)
            f.close()
            f = file(name, "w")
            f.write(secret)
            f.close()

    def secret(self):
        """Return the contents of the file 'secret'."""
        name = os.path.join(self._dotdir, "secret")
        f = file(name, "r")
        data = f.read()
        f.close()
        return data

    def create_list(self, listname):
        """Create a new MailingList."""
        self.create_dotdir()
        ml = eoc.ml.MailingList(self, listname)
        ml.create()
        return ml
        
    def open_list(self, listname):
        """Open an existing mailing list."""
        return eoc.hooks.open_list(self, listname)

    def find_lists(self):
        """Return list of all list names."""
        lists = []
        eoc.hooks.find_lists(self, lists)
        return lists

    def read_and_process_incoming(self):
        """Read an incoming message and process it appropriately."""
        tuple = eoc.hooks.read_incoming()
        if tuple is None:
            logging.error("No message was incoming.")
            return
        msg, sender, recipient = tuple
        logging.debug("Incoming message (from '%s' to '%s'):\n%s" %
                      (sender, recipient, msg.as_string()))

        ap = eoc.ap.AddressParser(self.find_lists(), self._commands)
        (list_name, parts) = ap.parse(recipient)
        ml = eoc.hooks.open_list(self, list_name)
        ml.process_incoming(msg, sender, parts)

    def add_address_command(self, command, has_hash, callback):
        """Add a new address command or a new callback to one."""
        self._commands.add_command(command, has_hash, callback)
