# Copyright 2005 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""Plugins and plugin manager."""


import imp
import logging
import os
import sys


import eoc.defaults
import eoc.exceptions


class WrongVersion(eoc.exceptions.EocException):

    """Exception thrown when plugin wants an incompatible plugin API."""
    
    def __init__(self, wanted_version):
        self.str = "Plugin wants version %s, we have version %s." \
                    % (wanted_version, eoc.defaults.PLUGIN_API_VERSION)


def _parse_api_version(version):
    parts = version.split(".")
    if len(parts) == 2:
        return int(parts[0]), int(parts[1])
    else:
        return None, None


def require_version(version):
    """Check that 'version' is compatible with the current API version.
    
    The plugin API version we support is defined in PLUGIN_API_VERSION.
    It consists of two integers, separated by periods: major and minor.
    Whenever we add a function, add a constant, or make an other
    interface change that doesn't break backwards compatibility, we
    increment the minor number. If we remove a function or constant, or
    change the meaning of a function or constant in a way that breaks
    plugins using the existing interface, we increment major and reset
    minor to zero.
    
    In other words, if we have API version "A.B", we're compatible if
    'version' is "a.b" such that a == A, b <= B. Examples:
    
        we have     plugin wants        result
        --------------------------------------
        0.0         0.0                 compatible
        0.1         0.0                 compatible
        0.0         0.1                 NOT compatible
        1.0         0.0                 NOT compatible
        0.0         1.0                 NOT compatible
        
    This allows eoc to be upgraded with backwards compatible changes
    without breaking plugins, and lets us notice if we upgrade plugins
    to versions that require a newer eoc than we have.
    
    If a mismatch is noticed, the WrongVersion exception is thrown.      
    """

    wanted_major, wanted_minor = _parse_api_version(version)
    current_major, current_minor = \
        _parse_api_version(eoc.defaults.PLUGIN_API_VERSION)

    if wanted_major == current_major and wanted_minor <= current_minor:
        pass
    else:
        raise WrongVersion(version)


class PluginManager:

    """Manage (locate, load) plugins."""
    
    def __init__(self):
        self._filenames = []
        self.add_file(os.path.join(eoc.defaults.SYSTEM_PLUGIN_DIR, 
                                   "default.py"))
        self.add_dir(eoc.defaults.SYSTEM_PLUGIN_DIR)
        
    def add_file(self, filename):
        """Add a file as a plugin, if it exists."""
        if os.path.exists(filename) and filename not in self._filenames:
            self._filenames.append(filename)
        
    def add_dir(self, dir):
        """Add a directory to the search path."""
        basenames = os.listdir(dir)
        basenames = [x for x in basenames if x.endswith(".py")]
        basenames.sort()
        for basename in basenames:
            self.add_file(os.path.join(dir, basename))
        
    def load(self):
        """Load all plugins in all directories."""
        for filename in self._filenames:
            logging.debug("Loading plugin: %s" % filename)
            basename = os.path.basename(filename)
            name, ext = os.path.splitext(basename)
            if name not in sys.modules:
                f = file(filename, "r")
                module = imp.load_module(name, f, filename, 
                                         (".py", "r", imp.PY_SOURCE))
                f.close()
