# Copyright 2005 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""Persistent storage."""


import ConfigParser
import logging
import os
import tempfile


import eoc.exceptions


class UnlockedStore(eoc.exceptions.EocException):

    """A Store is unlocked but a method that requires lock is used."""

    def __init__(self, object):
        self.str = "%s is unlocked, when it should be locked" % repr(object)


class Store:

    """Base class for Store implementations. Don't instantiate this one."""

    def __init__(self):
        self._locked = False
        self._lock_file_name = None
        
    def _set_defaults(self, defaults):
        if defaults:
            for itemname, fields in defaults.iteritems():
                for fieldname, value in fields.iteritems():
                    self._set_default_value(itemname, fieldname, value)

    def reload(self):
        """Re-load contents, in case they've changed on-disk."""
        pass

    def get_items(self):
        """Return list of keys for items."""
        return []
        
    def get_fields(self, itemkey):
        """Return list of non-empty fields in an item."""
        return []
        
    def get_value(self, itemkey, fieldname):
        """Return value of a field in an item ("" if not there)."""
        return ""
        
    def lock(self):
        """Lock store for modification. Raise LockFailure on error."""
        pass
        
    def assert_locked(self):
        """Make sure the store is locked. Throw UnlockedStore otherwise."""
        if not self._locked:
            raise UnlockedStore("%s is unlocked" % repr(self))

    def _set_default_value(self, itemkey, fieldname, value):
        """Set the default value of an item in an UNLOCKED store."""
        pass

    def set_value(self, itemkey, fieldname, value):
        """Change value of field in an item in a locked store."""
        pass
     
    def remove_item(self, itemkey):
        """Remove an item from a locked store."""
        pass
        
    def commit(self):
        """Save any changes to a locked store and unlock it."""
        pass

    def _create_lock_file(self):
        """Create a lock file."""
        # Note that this does not work on NFS, probably. Patch welcome. --liw
        assert self._lock_file_name is not None # Remember to set it!
        dirname = os.path.dirname(self._lock_file_name)
        (fd, pathname) = tempfile.mkstemp(suffix=".lock", dir=dirname)
        os.close(fd)
        while 1:
            try:
                os.link(pathname, self._lock_file_name)
                break
            except os.error:
                logging.debug("Failed to create lock %s, sleeping..." % 
                              self._lock_file_name)
                time.sleep(1)
        os.remove(pathname)
        logging.debug("Created lock %s" % self._lock_file_name)
        
    def _remove_lock_file(self):
        """Remove a lock file."""
        assert self._lock_file_name is not None # Remember to set it!
        os.remove(self._lock_file_name)
        logging.debug("Removed lock %s" % self._lock_file_name)


class ConfigParserStore(Store):

    """A Store that uses a ConfigParser file for storing data."""

    def __init__(self, filename, defaults=None):
        Store.__init__(self)
        self.filename = filename
        self._lock_file_name = filename + ".lock"
        self.cp = None
        self._set_defaults(defaults)
        
    def _load(self, force=False):
        if not self.cp or force:
            if not self.cp:
                self.cp = ConfigParser.ConfigParser()
            self.cp.read(self.filename)
            
    def reload(self):
        self._load(force=True)

    def get_items(self):
        self._load()
        return self.cp.sections()

    def get_fields(self, itemkey):
        self._load()
        return self.cp.options(itemkey)

    def get_value(self, itemkey, fieldname):
        self._load()
        try:
            return self.cp.get(itemkey, fieldname)
        except ConfigParser.NoSectionError:
            return ""
    
    def lock(self):
        self._create_lock_file()
        self._locked = True
        self._load(force=True)
        
    def _unlocked_set_value(self, itemkey, fieldname, value):
        if not self.cp.has_section(itemkey):
            self.cp.add_section(itemkey)
        self.cp.set(itemkey, fieldname, value)

    def _set_default_value(self, itemkey, fieldname, value):
        self._load()
        self._unlocked_set_value(itemkey, fieldname, value)

    def set_value(self, itemkey, fieldname, value):
        self.assert_locked()
        self._unlocked_set_value(itemkey, fieldname, value)

    def remove_item(self, itemkey):
        self.assert_locked()
        self.cp.remove_section(itemkey)
        
    def commit(self):
        self.assert_locked()
        f = file(self.filename + ".new", "w")
        self.cp.write(f)
        f.close()
        os.rename(self.filename + ".new", self.filename)
        self._remove_lock_file()
        self._locked = False
