#!/usr/bin/python
#
# Copyright 2005 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""Unit tests for eoc.py."""


import ConfigParser
import StringIO
import unittest
import os
import tempfile
import re
import shutil
import email.Message

import eoc


# Dot directory used by tests.
DOTDIR = "dot-eoc-testing"


class RequireVersionTests(unittest.TestCase):

    cases = [
        # api, wanted, compatible?
        ("0.0", "0.0", True),
        ("0.1", "0.0", True),
        ("0.0", "0.1", False),
        ("1.0", "0.0", False),
        ("1.0", "2.0", False),
    ]

    def testRequireVersion(self):
        real_version = eoc.defaults.PLUGIN_API_VERSION
        for api, wanted, is_compatible in self.cases:
            eoc.defaults.PLUGIN_API_VERSION = api
            try:
                eoc.plugins.require_version(wanted)
            except eoc.plugins.WrongVersion:
                self.failIf(is_compatible, 
                            "eoc.require_version says api version " + api +
                            " does not match " + wanted + 
                            " which plugin wants")
            else:
                self.failUnless(is_compatible,
                                "eoc.require_version says api version " + 
                                api + " matches " + wanted + 
                                " which plugin wants")
        eoc.defaults.PLUGIN_API_VERSION = real_version


class HookTests(unittest.TestCase):

    def having_fun_cb_1(self, ret):
        self.cb_count += 1
        return ret

    def having_fun_cb_2(self, ret):
        return self.having_fun_cb_1(ret)

    def testQueryHook(self):
        having_fun_yet_hook = eoc.hook.QueryHook(None)
        having_fun_yet_hook.add(self.having_fun_cb_1)
        having_fun_yet_hook.add(self.having_fun_cb_2)

        self.cb_count = 0
        self.failUnlessEqual(having_fun_yet_hook(True), True)
        self.failUnlessEqual(self.cb_count, 1)

        self.cb_count = 0
        self.failUnlessEqual(having_fun_yet_hook(False), False)
        self.failUnlessEqual(self.cb_count, 1)

        self.cb_count = 0
        self.failUnlessEqual(having_fun_yet_hook(None), None)
        self.failUnlessEqual(self.cb_count, 2)

        self.cb_count = 0
        having_fun_yet_hook.remove(self.having_fun_cb_1)
        self.failUnlessEqual(having_fun_yet_hook(None), None)
        self.failUnlessEqual(self.cb_count, 1)

    def hook_one(self, value):
        return 1
        
    def hook_two(self, value):
        return 2

    def testHookOrder(self):
        hook = eoc.hook.QueryHook(None)
        hook.add(self.hook_one)
        hook.add(self.hook_two)
        self.failUnlessEqual(hook(0), 2)

    def testHooks(self):
        self.failUnless(eoc.hooks.nanonano(True))


class PluginManagerTests(unittest.TestCase):

    def testPluginManager(self):
        pm = eoc.plugins.PluginManager()
        pm.load()
        self.failUnlessEqual(eoc.hooks.nanonano(105), 105)


class ConfigParserStoreTests(unittest.TestCase):

    input_file = """\
[foo]
bar = 1
foobar = abc, def

[bar]
bar = 2
foobar = heippatirallaa
"""

    def testDefaults(self):
        defaults = {
            "foo": {
                "bar": "1",
                "foobar": "abc, def",
            },
            "bar": {
                "bar": "2",
                "foobar": "heippatirallaa",
            }
        }
    
        (fd, filename) = tempfile.mkstemp()
        os.close(fd)
    
        store = eoc.store.ConfigParserStore(filename, defaults)
        
        default_items = defaults.keys()
        default_items.sort()
        store_items = store.get_items()
        store_items.sort()
        self.failUnlessEqual(default_items, store_items)
        
        for item, fields in defaults.iteritems():
            default_fields = fields.keys()
            default_fields.sort()
            store_fields = store.get_fields(item)
            store_fields.sort()
            self.failUnlessEqual(default_fields, store_fields)
            for field, value in fields.iteritems():
                self.failUnlessEqual(value, store.get_value(item, field))

    def testIt(self):
        (fd, filename) = tempfile.mkstemp()
        os.close(fd)
    
        store = eoc.store.ConfigParserStore(filename)
        store.cp = ConfigParser.ConfigParser()
        store.cp.readfp(StringIO.StringIO(self.input_file))
        
        self.failUnlessEqual(store.get_items(), ["foo", "bar"])
        self.failUnlessEqual(store.get_fields("foo"), ["bar", "foobar"])
        self.failUnlessEqual(store.get_value("foo", "bar"), "1")
        
        self.failUnlessRaises(eoc.store.UnlockedStore,
                              store.set_value,
                              "foo", "bar", "2")
        
        store.lock()
        store.set_value("foo", "bar", "2")
        store.commit()
        
        self.failUnlessEqual(store.get_value("foo", "bar"), "2")
        
        self.failUnlessRaises(eoc.store.UnlockedStore,
                              store.remove_item,
                              "foo")
        
        store.lock()
        store.remove_item("foo")
        store.commit()
        
        self.failUnlessEqual(store.get_value("foo", "bar"), "")

        store.lock()
        store.set_value("foo", "bar", "3")
        store.commit()

        self.failUnlessEqual(store.get_value("foo", "bar"), "3")


class AddressCleaningTestCases(unittest.TestCase):

    def setUp(self):
        commands = eoc.commands.AddressCommands()
        self.ap = eoc.ap.AddressParser(["foo@EXAMPLE.com",
                                        "bar@lists.example.com"],
                                       commands)
    
    def verify(self, address, wanted, skip_prefix=None, forced_domain=None):
        self.ap.set_skip_prefix(skip_prefix)
        self.ap.set_forced_domain(forced_domain)
        address = self.ap.clean(address)
        self.failUnlessEqual(address, wanted)
    
    def testSimpleAddress(self):
        self.verify("foo@example.com", "foo@example.com")

    def testUpperCaseAddress(self):
        self.verify("FOO@EXAMPLE.COM", "foo@example.com")

    def testPrefixRemoval(self):
        self.verify("foo@example.com", "foo@example.com", 
                    skip_prefix="user-")
        self.verify("user-foo@example.com", "foo@example.com", 
                    skip_prefix="user-")

    def testForcedDomain(self):
        self.verify("foo@example.com", "foo@example.com",
                    forced_domain="example.com")
        self.verify("foo@whatever.example.com", "foo@example.com", 
                    forced_domain="example.com")

    def testPrefixRemovalWithForcedDomain(self):
        self.verify("foo@example.com", "foo@example.com", 
                    skip_prefix="user-",
                    forced_domain="example.com")
        self.verify("foo@whatever.example.com", "foo@example.com", 
                    skip_prefix="user-",
                    forced_domain="example.com")
        self.verify("user-foo@example.com", "foo@example.com", 
                    skip_prefix="user-",
                    forced_domain="example.com")
        self.verify("user-foo@whatever.example.com", "foo@example.com", 
                    skip_prefix="user-",
                    forced_domain="example.com")

class AddressParserTestCases(unittest.TestCase):

    def setUp(self):
        pm = eoc.plugins.PluginManager()
        pm.load()
        commands = eoc.commands.AddressCommands()
        commands.add_command("subscribe", False, None)
        commands.add_command("bounce", True, None)
        self.ap = eoc.ap.AddressParser(["foo@EXAMPLE.com",
                                        "bar@lists.example.com"],
                                       commands)
    
    def verify_parser(self, address, wanted_listname, wanted_parts):
        listname, parts = self.ap.parse(address)
        self.failUnlessEqual(listname, wanted_listname)
        self.failUnlessEqual(parts, wanted_parts)
        
    def testParser(self):
        self.verify_parser("foo@example.com", 
                           "foo@EXAMPLE.com", 
                           [])
        self.verify_parser("foo-subscribe@example.com", 
                           "foo@EXAMPLE.com", 
                           ["subscribe"])
        self.verify_parser("foo-subscribe-joe=example.com@example.com", 
                           "foo@EXAMPLE.com", 
                           ["subscribe", "joe=example.com"])
        self.verify_parser("foo-bounce-123-ABCDEF@example.com", 
                           "foo@EXAMPLE.com", 
                           ["bounce", "123", "abcdef"])



class MailingListManagerTests(unittest.TestCase):

    LISTNAME = "announce@example.com"
    
    def name(self, relative_name):
        return os.path.join(DOTDIR, relative_name)

    def setUp(self):
        if os.path.exists(DOTDIR):
            shutil.rmtree(DOTDIR)

    def tearDown(self):
        if os.path.exists(DOTDIR):
            shutil.rmtree(DOTDIR)

    def testDotDir(self):
        self.failIf(os.path.exists(DOTDIR))

        mlm = eoc.mlm.MailingListManager(DOTDIR)
        self.failUnlessEqual(mlm._dotdir, DOTDIR)
        self.failIf(os.path.exists(DOTDIR))
        
        mlm.create_dotdir()
        self.failUnless(os.path.isdir(DOTDIR))
        self.failUnless(os.path.isfile(self.name("secret")))
        
    def testCreateList(self):
        mlm = eoc.mlm.MailingListManager(DOTDIR)
        mlm.create_dotdir()
        self.failIf(os.path.exists(self.name(self.LISTNAME)))

        mlm.create_list(self.LISTNAME)
        self.failUnless(os.path.isdir(self.name(self.LISTNAME)))
        dirname = self.name(self.LISTNAME)

        config_name = os.path.join(dirname, "config")
        self.failUnless(os.path.isfile(config_name))
        
        cp = ConfigParser.ConfigParser()
        cp.read([config_name])
        self.failUnlessEqual(cp.sections(), ["list"])
        
        subscribers_name = os.path.join(dirname, "subs")
        self.failUnless(os.path.isfile(subscribers_name))


class MailingListTestsBase(unittest.TestCase):

    LISTNAME = "staff@example.com"

    def setUp(self):
        if os.path.exists(DOTDIR):
            shutil.rmtree(DOTDIR)
        self.mlm = eoc.mlm.MailingListManager(DOTDIR)
        self.ml = self.mlm.create_list(self.LISTNAME)
        self.sent_mail = []
        eoc.hooks.send_mail.add(self.send_mail)
        eoc.hooks.read_incoming.add(self.read_incoming)

    def tearDown(self):
        if os.path.exists(DOTDIR):
            shutil.rmtree(DOTDIR)
        self.mlm = None
        self.ml = None
        eoc.hooks.send_mail.remove(self.send_mail)
        eoc.hooks.send_mail.remove(self.read_incoming)

    def send_mail(self, msg, sender, recipients):
        self.sent_mail.append((msg, sender, recipients))
        return True

    def read_incoming(self):
        return self.msg, self.sender, self.recipient

    def mail_transaction(self, sender, recipient):
        self.sender = sender
        self.recipient = recipient
        
        self.msg = email.Message.Message()
        self.msg["From"] = self.sender
        self.msg["To"] = self.recipient
        
        self.sent_mail = []
        self.mlm.read_and_process_incoming()

    def match(self, sender_re, recipient_re, header_re):
        sender_pat = re.compile(sender_re)
        recipient_pat = re.compile(recipient_re)
        header_pat = re.compile(header_re, re.M)
        
        for i in range(len(self.sent_mail)):
            (msg, sender, recipients) = self.sent_mail[i]
            sender_m = sender_pat.match(sender)
            headers = msg.as_string().split("\n\n", 1)[0]
            header_m = header_pat.search(headers)
            
            for recipient in recipients:
                recipient_m = recipient_pat.match(recipient)
                if sender_m and recipient_m and header_m:
                    del self.sent_mail[i]
                    return header_m
                
        self.fail("No matching messages.")
            

class MailingListTests(MailingListTestsBase):

    def testCommandAddress(self):
        self.failUnlessEqual(self.ml.command_address("subscribe", []),
                             "staff-subscribe@example.com")
        self.failUnlessEqual(self.ml.command_address("subscribe", 
                                                     ["foo=example.com"]),
                             "staff-subscribe-foo=example.com@example.com")

    def testHashedCommandAddress(self):
        hashed = self.ml.hashed_command_address("foo")
        parts = hashed.split("@", 1)[0].split("-")[2:]
        self.failUnless(self.ml.hashed_is_ok("foo", parts))
        
        self.failIf(self.ml.hashed_is_ok("bar", parts))
        self.failIf(self.ml.hashed_is_ok("foo", parts[:-1] + ["abc"]))

    def testSubscribeAndUnsubscribe(self):
        self.failUnlessEqual(self.ml.subscriberdb.list(), [])
        self.failIf(self.ml.subscriberdb.has("gytha@example.com"))
        
        self.ml.subscriberdb.add("gytha@example.com")

        self.failUnlessEqual(self.ml.subscriberdb.list(), 
                             ["gytha@example.com"])
        self.failUnless(self.ml.subscriberdb.has("gytha@example.com"))
        
        self.ml.subscriberdb.remove("gytha@example.com")

        self.failUnlessEqual(self.ml.subscriberdb.list(), [])
        self.failIf(self.ml.subscriberdb.has("gytha@example.com"))


class IncomingPostsTests(MailingListTestsBase):

    def testIncomingPost(self):
        subscriber = "rincewind@example.com"
        self.ml.subscriberdb.add(subscriber)
    
        self.mail_transaction("user@example.com", "staff@example.com")

        self.failUnless(len(self.sent_mail) == 1)

        (sent_msg, sent_sender, sent_recipients) = self.sent_mail[0]
        for header in self.msg.keys():
            self.failUnlessEqual(self.msg[header], sent_msg[header])

        self.failUnlessEqual(sent_sender, "staff-ignore@example.com")
        self.failUnlessEqual(sent_recipients, [subscriber])


class IncomingSubscriptionTests(MailingListTestsBase):

    def testIncomingSubscription(self):
        self.mail_transaction("user@example.com", 
                              "staff-subscribe@example.com")
        m = self.match("staff-ignore@example.com",
                       "user@example.com",
                       "^From: (staff-subyes-.+-[0-9a-f]{32}@example.com)")
        self.failUnlessEqual(self.sent_mail, [])
        self.ml.subscriberdb.reload()
        self.failUnlessEqual(self.ml.subscriberdb.list(), [])
        
        self.mail_transaction("otheruser@example.com",
                              m.group(1))
        self.match("staff-ignore@example.com",
                   "user@example.com",
                   "^Subject: Welcome")
        self.failUnlessEqual(self.sent_mail, [])
        self.ml.subscriberdb.reload()
        self.failUnlessEqual(self.ml.subscriberdb.list(), 
                             ["user@example.com"])
        
        
if __name__ == "__main__":
    if True:
        eoc.log.setup([])
    unittest.main()
