# Copyright 2006 Lars Wirzenius (liw@iki.fi)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA



"""Default callbacks for EoC plugins

This module contains default callbacks for all of EoC plugins. The
plugin manager is set up to load this plugin before any other.

"""


import email
import email.MIMEText
import logging
import os
import sys


import eoc


LIST_DIR_MODE = 0700


def setup_system_options():
    eoc.options.parser.add_option("-d", "--debug", 
                                  action="callback",
                                  callback=eoc.log.set_stdout_debug,
                                  help="Put all debugging info to stdout, " +
                                        "not just log files.")

    eoc.options.parser.add_option("--dotdir",
                                  action="store",
                                  default=\
                                    os.path.expanduser(eoc.defaults.DOTDIR),
                                  help="Use DOTDIR for storing EoC files." +
                                        "(Currently " +
                                        eoc.defaults.DOTDIR + 
                                        ")")

    eoc.options.parser.add_option("--name",
                                  action="store",
                                  help="Give name of list being manipulated.")

    eoc.options.parser.add_option("--create", 
                                  action="store_true",
                                  default=False,
                                  help="Create a list named with --name")

    eoc.options.parser.add_option("--subscribe", 
                                  action="store_true",
                                  default=False,
                                  help="Subscribe addresses given on " +
                                       "command line.")

    eoc.options.parser.add_option("--unsubscribe", 
                                  action="store_true",
                                  default=False,
                                  help="Unsubscribe addresses given on " +
                                       "command line.")

    eoc.options.parser.add_option("--incoming", 
                                  action="store_true",
                                  default=False,
                                  help="Handle an incoming message.")


class ListNameRequired(eoc.exceptions.EocException):

    def __init__(self):
        self.str = "List name not specified with --name option."


def require_name(options):
    if not options.name:
        raise ListNameRequired()


class InternalError(eoc.exceptions.EocException):

    def __init__(self, msg):
        self.str = "Internal error: " + msg


def handle_options(mlm, options, args):
    if options.create:
        require_name(options)
        ml = mlm.create_list(options.name)
        return
    
    if options.subscribe:
        require_name(options)
        ml = mlm.open_list(options.name)
        for addr in args:
            ml.subscriberdb.add(addr)
            logging.info("Subscribed %s to %s" % (addr, ml._name))
        return
    
    if options.unsubscribe:
        require_name(options)
        ml = mlm.open_list(options.name)
        for addr in args:
            ml.subscriberdb.remove(addr)
            logging.info("Unsubscribed %s to %s" % (addr, ml._name))
        return
        
    if options.incoming:
        mlm.read_and_process_incoming()
        return
        
    raise InternalError("Don't actually know what you want to do")


def obey_subscribe(mlm, ml, sender, parts, msg):
    if parts:
        addr = "-".join(parts)
    else:
        addr = sender

    cmd_addr = ml.hashed_command_address("subyes", [addr])
    
    msg = email.MIMEText.MIMEText("Reply to confirm. Otherwise ignore.")
    msg["From"] = cmd_addr
    msg["To"] = addr
    msg["Subject"] = "Confirm subscription to " + ml._name
    
    eoc.hooks.send_mail(msg, ml.command_address("ignore"), [addr])


def obey_subyes(mlm, ml, sender, parts, msg):
    if not ml.hashed_is_ok("subyes", parts):
        logging.warning("Invalid hash in command address: %s" % parts)
        return True

    addr = "@".join(parts[0].split("="))
    ml.subscriberdb.add(addr)
    
    msg = email.MIMEText.MIMEText("Welcome to %s." % ml._name)
    msg["From"] = ml.command_address("help")
    msg["To"] = addr
    msg["Subject"] = "Welcome to " + ml._name
    
    eoc.hooks.send_mail(msg, ml.command_address("ignore"), [addr])
    

def created_list_manager(mlm):
    mlm.add_address_command("ignore", False, lambda *args: None)
    mlm.add_address_command("subscribe", False, obey_subscribe)
    mlm.add_address_command("subyes", True, obey_subyes)


def list_dirname(mlm, ml):
    """Return name of list's directory."""
    return os.path.join(mlm._dotdir, ml._name)


def list_relative(mlm, ml, filename):
    """Return pathname to file in list's direcory."""
    return os.path.join(list_dirname(mlm, ml), filename)


def find_lists(mlm, lists):
    for name in os.listdir(mlm._dotdir):
        if "@" in name:
            lists.append(name)
    return False


def list_exists(mlm, listname):
    return os.path.exists(os.path.join(mlm._dotdir, listname))


def create_list(mlm, ml):
    os.mkdir(list_dirname(mlm, ml), LIST_DIR_MODE)
    ml.config = eoc.store.ConfigParserStore(list_relative(mlm, ml, "config"),
                                            eoc.defaults.DEFAULT_LIST_CONFIG)
    ml.config.lock()
    ml.config.commit()
    ml.subscriberdb = \
       eoc.subscriberdb.SubscriberDB(
        eoc.store.ConfigParserStore(list_relative(mlm, ml, "subs")))


def read_incoming():
    data = sys.stdin.read()
    if not data:
        return None
    msg = email.message_from_string(data)
    return msg, os.environ["SENDER"], os.environ["RECIPIENT"]


def send_mail(msg, sender, recipients):
    if eoc.defaults.sendmail:
        logging.debug("Sending mail from %s to %s:\n%s" % 
                      (sender, ", ".join(recipients), msg.as_string()))
        recipients = " ".join(recipients)
        f = os.popen("%s -oi -f '%s' %s" %
                        (eoc.defaults.sendmail, sender, recipients),
                     "w")
        f.write(msg.as_string())
        f.close()
    else:
        logging.debug("Pretending to send mail from %s to %s:\n%s" % 
                      (sender, 
                       ", ".join(recipients), 
                       msg.as_string(unixfrom=True)))


def open_list(mlm, listname):
    logging.debug("Opening list '%s'" % listname)
    listdir = os.path.join(mlm._dotdir, listname)
    if not os.path.isdir(listdir):
        raise eoc.ml.ListDoesNotExist(listname)
    ml = eoc.ml.MailingList(mlm, listname)
    ml.config = eoc.store.ConfigParserStore(list_relative(mlm, ml, "config"))
    store = eoc.store.ConfigParserStore(list_relative(mlm, ml, "subs"))
    ml.subscriberdb = eoc.subscriberdb.SubscriberDB(store)
    return ml


def init():
    # The "nanonano" hook is used by unit testing.
    eoc.hooks.nanonano.add(lambda value: value)

    eoc.hooks.setup_option_parser.add(setup_system_options)
    eoc.hooks.handle_options.add(handle_options)
    eoc.hooks.created_list_manager.add(created_list_manager)
    eoc.hooks.find_lists.add(find_lists)
    eoc.hooks.list_exists.add(list_exists)
    eoc.hooks.create_list.add(create_list)
    eoc.hooks.read_incoming.add(read_incoming)
    eoc.hooks.send_mail.add(send_mail)
    eoc.hooks.open_list.add(open_list)


init()
